//
// Copyright (c) 2002
// Ronald Kevin Burton
//
// This material is provided "as is", with absolutely no warranty expressed
// or implied. This software is provided solely as part of ".NET Common Language
// Runtime Unleashed" to help illustrate concepts and principles described in 
// that book. Any use is at your own risk.
//
// Permission to use or copy this software for any purpose is hereby granted 
// without fee, provided the above notices are retained on all copies.
// Permission to modify the code and to distribute modified code is granted,
// provided the above notices are retained, and a notice that the code was
// modified is included with the above copyright notice.
//
// 
using System;
using System.Runtime.InteropServices;

namespace COMInteropSamples
{
	/// <summary>
	/// Summary description for Arithmetic.
	/// </summary>
	[StructLayout(LayoutKind.Sequential)]
	public struct Complex 
	{
		public double real;
		public double imaginary;
		public void Polar(out double r, out double t)
		{ 
			r = Math.Sqrt(real*real + imaginary * imaginary);
			t = Math.Atan2(imaginary, real);
		}
	}
	public enum ArithmeticErrors
	{
		DivideByZero = 0,
		Overflow,
		Underflow
	}

	[InterfaceTypeAttribute(ComInterfaceType.InterfaceIsIUnknown)]
	public interface IBasicOperations
	{
		int Add(int a, int b);
		int Subtract(int a, int b);
		int Multiply(int a, int b);
		int Divide(int a, int b);
		Complex Add(Complex a, Complex b);
		ArithmeticErrors LastError
		{
			get;
		}
	}

	[ClassInterface(ClassInterfaceType.None)]
	public class Arithmetic : IBasicOperations
	{
		ArithmeticErrors error;
		public Complex Add(Complex a, Complex b)
		{
			Complex c;
			c.real = a.real + b.real;
			c.imaginary = a.imaginary + b.imaginary;
			return c;
		}
		public ArithmeticErrors LastError
		{
			get
			{
				return error;
			}
		}
		public int Add(int a, int b)
		{
			return a + b;
		}
		public int Subtract(int a, int b)
		{
			return a - b;
		}
		public int Multiply(int a, int b)
		{
			return a * b;
		}
		public int Divide(int a, int b)
		{
			if(b != 0)
				return a / b;
			else
			{
				error = ArithmeticErrors.DivideByZero;
				return 0;
			}
		}
	}
}
