//
// Copyright (c) 2002
// Ronald Kevin Burton
//
// This material is provided "as is", with absolutely no warranty expressed
// or implied. This software is provided solely as part of ".NET Common Language
// Runtime Unleashed" to help illustrate concepts and principles described in 
// that book. Any use is at your own risk.
//
// Permission to use or copy this software for any purpose is hereby granted 
// without fee, provided the above notices are retained on all copies.
// Permission to modify the code and to distribute modified code is granted,
// provided the above notices are retained, and a notice that the code was
// modified is included with the above copyright notice.
//
// 
using System;
using System.Collections;
using System.Runtime.InteropServices;

using System.IO;
using System.Reflection;
using System.Resources;
using System.Security;
using System.Security.Cryptography;
using System.Threading;
using System.Runtime.Remoting;
using System.Runtime.Serialization;
using System.Diagnostics;
using System.Drawing;

namespace InteropSample
{
	[ComVisible(false)]
	public delegate void ColorHandler(Color o);
	[ComVisible(false)]
	public delegate void DateHandler(DateTime dt);
	[ComVisible(false)]
	public delegate void NameHandler(string s);
	[ComVisible(false)]
	public delegate void DecimalHandler(decimal d);
	[ComVisible(false)]
	public delegate void VariantHandler(object o);
	[ComVisible(false)]
	public delegate void ArrayHandler(object [] o);
	[ComVisible(false)]
	public delegate void CounterHandler(int c);

	[InterfaceType(ComInterfaceType.InterfaceIsIDispatch)]
	public interface _IComTypesEvents
	{
		[DispId(1)]
		void ColorChanged(Color o);
		[DispId(2)]
		void DateChanged(DateTime d);
		[DispId(3)]
		void NameChanged(string s);
		[DispId(4)]
		void DecimalChanged(decimal d);
		[DispId(5)]
		void VariantChanged(object o);
		[DispId(6)]
		void ArrayChanged(object [] o);
		[DispId(7)]
		void Counter(int c);
	}

	public interface IComTypes
	{
		Color ColorValue
		{
			get;
			set;
		}
		DateTime Date
		{
			get;
			set;
		}
		string Name
		{
			get;
			set;
		}
		decimal Decimal
		{
			get;
			set;
		}
		object Variant
		{
			get;
			set;
		}
		object [] Array
		{
			get;
			set;
		}
		IEnumerator Enumerator
		{
			get;
		}

		void ErrorTest(string error);
	}
	/// <summary>
	/// Summary description for CComTypes.
	/// </summary>
	[ClassInterface(ClassInterfaceType.None)]
	[ComSourceInterfaces("InteropSample._IComTypesEvents, dotNETCOMTypes")]
	public class CComTypes : IComTypes
	{
		public event ColorHandler ColorChanged;
		public event DateHandler DateChanged;
		public event NameHandler NameChanged;
		public event DecimalHandler DecimalChanged;
		public event VariantHandler VariantChanged;
		public event ArrayHandler ArrayChanged;
		public event CounterHandler Counter;
		Hashtable errorExceptions;
		void CounterEntry()
		{
			try
			{
				while(true)
				{
					Thread.Sleep(1000);
					if(Counter != null)
						Counter(counter++);
				}
			}
			finally
			{
				Debug.WriteLine("Exiting counter thread!");
			}
		}
		public CComTypes()
		{
			counter = 0;

			counterThread = new Thread(new ThreadStart(CounterEntry));
			counterThread.Start();

			errorExceptions = new Hashtable();
			errorExceptions.Add("COR_E_APPLICATION", new ApplicationException("This is a test"));
			errorExceptions.Add("COR_E_ARGUMENT", new ArgumentException("This is a test"));
			errorExceptions.Add("E_INVALIDARG", new ArgumentException("This is a test"));
			errorExceptions.Add("COR_E_ARGUMENTOUTOFRANGE", new ArgumentOutOfRangeException("This is a test"));
			errorExceptions.Add("COR_E_ARITHMETIC", new ArithmeticException("This is a test"));
			errorExceptions.Add("ERROR_ARITHMETIC_OVERFLOW", new ArithmeticException("This is a test"));
			errorExceptions.Add("COR_E_ARRAYTYPEMISMATCH", new ArrayTypeMismatchException("This is a test"));
			errorExceptions.Add("COR_E_BADIMAGEFORMAT", new BadImageFormatException("This is a test"));
			errorExceptions.Add("ERROR_BAD_FORMAT", new BadImageFormatException("This is a test"));
			errorExceptions.Add("COR_E_CONTEXTMARSHAL", new ContextMarshalException("This is a test"));
			errorExceptions.Add("NTE_FAIL", new CryptographicException("This is a test"));
			errorExceptions.Add("COR_E_DIRECTORYNOTFOUND", new DirectoryNotFoundException("This is a test"));
			errorExceptions.Add("ERROR_PATH_NOT_FOUND", new DirectoryNotFoundException("This is a test"));
			errorExceptions.Add("COR_E_DIVIDEBYZERO", new DivideByZeroException("This is a test"));
			errorExceptions.Add("COR_E_DUPLICATEWAITOBJECT", new DuplicateWaitObjectException("This is a test"));
			errorExceptions.Add("COR_E_ENDOFSTREAM", new EndOfStreamException("This is a test"));
			// errorExceptions.Add("COR_E_TYPELOAD", new EntryPointNotFoundException("This is a test"));
			errorExceptions.Add("COR_E_EXCEPTION", new Exception("This is a test"));
			errorExceptions.Add("COR_E_EXECUTIONENGINE", new ExecutionEngineException("This is a test"));
			errorExceptions.Add("COR_E_FIELDACCESS", new FieldAccessException("This is a test"));
			errorExceptions.Add("COR_E_FILENOTFOUND", new FileNotFoundException("This is a test"));
			errorExceptions.Add("ERROR_FILE_NOT_FOUND", new FileNotFoundException("This is a test"));
			errorExceptions.Add("COR_E_FORMAT", new FormatException("This is a test"));
			errorExceptions.Add("COR_E_INDEXOUTOFRANGE", new IndexOutOfRangeException("This is a test"));
			errorExceptions.Add("COR_E_INVALIDCAST", new InvalidCastException("This is a test"));
			errorExceptions.Add("E_NOINTERFACE", new InvalidCastException("This is a test"));
			errorExceptions.Add("COR_E_INVALIDCOMOBJECT", new InvalidComObjectException("This is a test"));
			errorExceptions.Add("COR_E_INVALIDFILTERCRITERIA", new InvalidFilterCriteriaException("This is a test"));
			errorExceptions.Add("COR_E_INVALIDOLEVARIANTTYPE", new InvalidOleVariantTypeException("This is a test"));
			errorExceptions.Add("COR_E_INVALIDOPERATION", new InvalidOperationException("This is a test"));
			errorExceptions.Add("COR_E_IO", new IOException("This is a test"));
			errorExceptions.Add("COR_E_MEMBERACCESS", new MemberAccessException("This is a test"));
			errorExceptions.Add("COR_E_METHODACCESS", new MethodAccessException("This is a test"));
			errorExceptions.Add("COR_E_MISSINGFIELD", new MissingFieldException("This is a test"));
			errorExceptions.Add("COR_E_MISSINGMANIFESTRESOURCE", new MissingManifestResourceException("This is a test"));
			errorExceptions.Add("COR_E_MISSINGMEMBER", new MissingMemberException("This is a test"));
			errorExceptions.Add("COR_E_MISSINGMETHOD", new MissingMethodException("This is a test"));
			errorExceptions.Add("COR_E_MULTICASTNOTSUPPORTED", new MulticastNotSupportedException("This is a test"));
			errorExceptions.Add("COR_E_NOTFINITENUMBER", new NotFiniteNumberException("This is a test"));
			errorExceptions.Add("E_NOTIMPL", new NotImplementedException("This is a test"));
			errorExceptions.Add("COR_E_NOTSUPPORTED", new NotSupportedException("This is a test"));
			errorExceptions.Add("COR_E_NULLREFERENCE", new NullReferenceException("This is a test"));
			errorExceptions.Add("E_POINTER", new NullReferenceException("This is a test"));
			errorExceptions.Add("COR_E_OUTOFMEMORY", new OutOfMemoryException("This is a test"));
			errorExceptions.Add("E_OUTOFMEMORY", new OutOfMemoryException("This is a test"));
			errorExceptions.Add("COR_E_OVERFLOW", new OverflowException("This is a test"));
			errorExceptions.Add("COR_E_PATHTOOLONG", new PathTooLongException("This is a test"));
			errorExceptions.Add("ERROR_FILENAME_EXCED_RANGE", new PathTooLongException("This is a test"));
			errorExceptions.Add("COR_E_RANK", new RankException("This is a test"));
			errorExceptions.Add("COR_E_REFLECTIONTYPELOAD", new COMException("This should be a ReflectionTypeLoadException"));
			errorExceptions.Add("COR_E_REMOTING", new RemotingException("This is a test"));
			errorExceptions.Add("COR_E_SAFEARRAYTYPEMISMATCH", new SafeArrayTypeMismatchException("This is a test"));
			errorExceptions.Add("COR_E_SECURITY", new SecurityException("This is a test"));
			errorExceptions.Add("COR_E_SERIALIZATION", new SerializationException("This is a test"));
			errorExceptions.Add("COR_E_STACKOVERFLOW", new StackOverflowException("This is a test"));
			errorExceptions.Add("ERROR_STACK_OVERFLOW", new StackOverflowException("This is a test"));
			errorExceptions.Add("COR_E_SYNCHRONIZATIONLOCK", new SynchronizationLockException("This is a test"));
			errorExceptions.Add("COR_E_SYSTEM", new SystemException("This is a test"));
			errorExceptions.Add("COR_E_TARGET", new TargetException("This is a test"));
			errorExceptions.Add("COR_E_TARGETINVOCATION", new COMException("This should be a TargetInvocationException"));
			errorExceptions.Add("COR_E_TARGETPARAMCOUNT", new TargetParameterCountException("This is a test"));
			errorExceptions.Add("COR_E_THREADABORTED", new COMException("This should be a ThreadAbortException"));
			errorExceptions.Add("COR_E_THREADINTERRUPTED", new ThreadInterruptedException("This is a test"));
			errorExceptions.Add("COR_E_THREADSTATE", new ThreadStateException("This is a test"));
			errorExceptions.Add("COR_E_THREADSTOP", new COMException("This should be a ThreadStopException"));
			errorExceptions.Add("COR_E_TYPELOAD", new TypeLoadException("This is a test"));
			errorExceptions.Add("COR_E_TYPEINITIALIZATION", new COMException("This should be a TypeInitializationException"));
			errorExceptions.Add("COR_E_VERIFICATION", new VerificationException("This is a test"));
			errorExceptions.Add("E_FAIL", new COMException("This is a test"));
		}
		public Color ColorValue
		{
			get
			{
				return colorValue;
			}
			set
			{
				colorValue = value;
				if(ColorChanged != null)
					ColorChanged(colorValue);
			}
		}
		public DateTime Date
		{
			get
			{
				return date;
			}
			set
			{
				date = value;
				if(DateChanged != null)
					DateChanged(date);
			}
		}
		public string Name
		{
			get
			{
				return name;
			}
			set
			{
				name = value;
				if(NameChanged != null)
					NameChanged(name);
			}
		}
		public decimal Decimal
		{
			get
			{
				return decimalValue;
			}
			set
			{
				decimalValue = value;
				if(DecimalChanged != null)
					DecimalChanged(decimalValue);
			}
		}
		public object Variant
		{
			get
			{
				return variantValue;
			}
			set
			{
				variantValue = value;
				if(VariantChanged != null)
					VariantChanged(variantValue);
			}
		}
		public object [] Array
		{
			get
			{
				return array;
			}
			set
			{
				array = value;
				if(ArrayChanged != null)
					ArrayChanged(array);
			}
		}
		public IEnumerator Enumerator
		{
			get
			{
				return array.GetEnumerator();
			}
		}
		public void ErrorTest(string error)
		{
			throw (Exception)errorExceptions[error];
		}
		Color colorValue;
		string name;
		decimal decimalValue;
		DateTime date;
		object variantValue;
		int counter;
		object [] array;
		Thread counterThread;
	}
}
